// /************************************************************
// *                                                           *
// *   SleekInventory                                          *
// *                                                           *
// *   Copyright (c) 2011 Rarebyte                             *
// *   http://www.rarebyte.com                                 *
// *                                                           *
// *   THIS SOURCE CODE IS PROPERTY OF Rarebyte. You may not   *
// *   copy, redistribute, modify, reuse or misuse this file   *
// *   in any way, except with permission of Rarebyte.         *
// *                                                           *
// ************************************************************/

using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SleekInventorySystem;

/// <summary>
/// Simple test class to show how to set up and use the sleek inventory system to interact with gameobjects.
/// </summary>
public sealed class TestInventoryController_SpawnItemInScene : MonoBehaviour, IInventoryPageGenerator {

  private SleekInventory testInventory = null;

  void Start() {

    //Fetch the SleekInventory component and call the CreatePages method with a class that implements the IInventoryPageGenerator interface as parameter.
    //In this example this TestInventoryController implements the necessary interface method.
    testInventory = this.gameObject.GetComponent<SleekInventory>();
    testInventory.CreatePages(this);

    //Register collider.
    ColliderDragTarget dragTarget = new ColliderDragTarget(GameObject.Find("/DropWall").collider, InventoryItemDroppedOnWall);
    testInventory.AddCustomDragTarget(dragTarget);
  }

  /// <summary>
  /// Inventory item dropping callback method.
  /// Note that this method must return whether or not the item should be removed from the inventory. E.g. whether or not the drop was succesful.
  /// </summary>
  private bool InventoryItemDroppedOnWall(AbstractDraggableItem draggableItem) {

    RaycastHit hit;
    if (Physics.Raycast(Camera.main.ScreenPointToRay(Input.mousePosition), out hit) == true) {
      GameObject spawnedGO = GameObject.Instantiate(GameObject.Find("/CubePrefab"), hit.point, Quaternion.Euler(10, 0, 180)) as GameObject; //Spawn cloned cube.
      spawnedGO.transform.parent = GameObject.Find("/SpawnedPrefabs").transform; //Set up a simple hierarchy to be able to reset (destroy) all spawned cubes at a later point.
      spawnedGO.rigidbody.AddRelativeTorque(Random.Range(-10, 10), Random.Range(-10, 10), Random.Range(-10, 10)); //Add a little torque so there's some motion to it.
      spawnedGO.renderer.material.mainTexture = (draggableItem as ImageItem).Icon; //Apply texture.
    }

    return (true);
  }

  public List<AbstractInventoryPage> GeneratePages() {

    //First: Create all necessary pages.
    List<AbstractInventoryPage> pages = new List<AbstractInventoryPage>();

    //Define the amount of inventory slots per page.
    int inventorySizeX = 5;
    int inventorySizeY = 4;

    //Add 1 page.
    pages.Add(new ItemGridPage(inventorySizeX, inventorySizeY));

    //Texture loading helper.
    SleekInventoryTextureContainer textureContainer = this.gameObject.GetComponent<SleekInventoryTextureContainer>();

    //Generate and assign items to specific pages of the inventory and to specific slots.
    //The index into the pages array defines the page of the item. The first 2 parameters of the SetSlotItem() method define the position in the inventory grid of the page.
    (pages[0] as ItemGridPage).SetSlotItem(0, 0, new ImageItem(textureContainer.GetTextureByFilename("test_icon_helmet")));
    (pages[0] as ItemGridPage).SetSlotItem(1, 0, new ImageItem(textureContainer.GetTextureByFilename("test_icon_potion")));
    (pages[0] as ItemGridPage).SetSlotItem(2, 0, new ImageItem(textureContainer.GetTextureByFilename("test_icon_sword")));
    (pages[0] as ItemGridPage).SetSlotItem(0, 1, new ImageItem(textureContainer.GetTextureByFilename("test_icon_potion")));
    (pages[0] as ItemGridPage).SetSlotItem(1, 1, new ImageItem(textureContainer.GetTextureByFilename("test_icon_sword")));
    (pages[0] as ItemGridPage).SetSlotItem(2, 1, new ImageItem(textureContainer.GetTextureByFilename("test_icon_helmet")));

    //Pages are created now so let's return them.
    return (pages);
  }

  void OnGUI() {
    GUI.color = Color.black;
    GUI.Label(new Rect(360, 30, 500, 25), "Drag items out to drop them in the scene.");
    GUI.color = Color.white;
    if (GUI.Button(new Rect(360, 55, 100, 25), "Reset") == true) {
      testInventory.CreatePages(this);
      foreach (Transform spawnedPrefab in GameObject.Find("/SpawnedPrefabs").transform) {
        GameObject.Destroy(spawnedPrefab.gameObject);
      }
    }
  }
}
